// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Collider.hpp"
#include "assert/assert.hpp"
#include "game/Game.hpp"

using namespace Dawn;
using namespace JPH;
using namespace JPH::literals;

void Collider::onInit() {
  assertNull(this->body, "Body is not NULL?");

  auto settings = this->getShapeSettings();
  auto shapeResult = settings->Create();
  auto shape = shapeResult.Get();

  auto pos = getItem()->getLocalPosition();

  BodyCreationSettings bodySettings(
    shape,
    RVec3(pos.x, pos.y, pos.z),
    Quat::sIdentity(),
    EMotionType::Dynamic,
    layer
  );

  this->body = getBodyInterface().CreateBody(bodySettings);
  assertNotNull(this->body, "Body failed to create?");
  this->bodyId = this->body->GetID();
  
  getBodyInterface().AddBody(this->bodyId, EActivation::Activate);
}

void Collider::onDispose() {
  getBodyInterface().RemoveBody(this->bodyId);
  getBodyInterface().DestroyBody(this->bodyId);
}

BodyInterface & Collider::getBodyInterface() {
  return getGame()->physicsManager->getBodyInterface();
}

BodyID Collider::getBodyId() {
  return bodyId;
}