// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItem.hpp"

namespace Dawn {
  enum class ColliderType {
    DYNAMIC,
    STATIC,
    KINEMATIC
  };

  class Collider : public SceneComponent {
    private:
      JPH::Body *body = nullptr;

    protected:
      JPH::BodyID bodyId;
      JPH::EMotionType emotionType = JPH::EMotionType::Dynamic;
      JPH::ObjectLayer layer = 1;

      /**
       * Returns the shape settings for the collider.
       * 
       * @return The shape settings for the collider.
       */
      virtual std::shared_ptr<JPH::ShapeSettings> getShapeSettings() = 0;

      /**
       * Returns the JoltPhysics body interface system.
       * 
       * @return The JoltPhysics body interface system.
       */
      JPH::BodyInterface & getBodyInterface();

    public:
      ColliderType type = ColliderType::KINEMATIC;

      void onInit() override;
      void onDispose() override;

      /**
       * Returns the JoltPhysics body ID of the collider.
       * 
       * @return The body ID of the collider.
       */
      JPH::BodyID getBodyId();
  };
}