
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"

#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "component/display/MeshRenderer.hpp"
#include "display/mesh/CubeMesh.hpp"
#include "component/physics/BoxCollider.hpp"

using namespace Dawn;

void Dawn::helloWorldScene(Scene &s) {
  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  cameraItem->lookAt({ 5, 5, 5 }, { 0, 0, 0 }, { 0, 1, 0 });
  camera->clipFar = 99999.99f;

  // Ground
  {
    // Create the scene item.
    auto cubeItem = s.createSceneItem();
    cubeItem->setLocalPosition(glm::vec3(0, 10, 0));

    // Create a simple cube mesh.
    auto cubeMesh = std::make_shared<Mesh>();
    cubeMesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
    CubeMesh::buffer(cubeMesh, glm::vec3(-1, -1, -1), glm::vec3(2, 2, 2), 0, 0);

    // Add a renderer to the scene item.
    auto cubeMeshRenderer = cubeItem->addComponent<MeshRenderer>();
    cubeMeshRenderer->mesh = cubeMesh;

    // Add a material to the scene item.
    auto cubeMaterial = cubeItem->addComponent<SimpleTexturedMaterial>();
    cubeMaterial->setColor(COLOR_MAGENTA);

    // Add collider
    auto boxCollider = cubeItem->addComponent<BoxCollider>();
  }

  {
    // Create the scene item.
    auto cubeItem = s.createSceneItem();
    cubeItem->setLocalPosition(glm::vec3(0.88f, 13, 0.1f));

    // Create a simple cube mesh.
    auto cubeMesh = std::make_shared<Mesh>();
    cubeMesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
    CubeMesh::buffer(cubeMesh, glm::vec3(-1, -1, -1), glm::vec3(2, 2, 2), 0, 0);

    // Add a renderer to the scene item.
    auto cubeMeshRenderer = cubeItem->addComponent<MeshRenderer>();
    cubeMeshRenderer->mesh = cubeMesh;

    // Add a material to the scene item.
    auto cubeMaterial = cubeItem->addComponent<SimpleTexturedMaterial>();
    cubeMaterial->setColor(COLOR_MAGENTA);

    // Add collider
    auto boxCollider = cubeItem->addComponent<BoxCollider>();
  }
}